﻿Imports System.Diagnostics

Namespace System.Drawing

   <DebuggerDisplay("({ToString()})")> _
   Public Structure RectF : Implements IEnumerable(Of VectorF)

      Public Near, Far As VectorF
      Public Sub New(ByVal Near As VectorF, ByVal Far As VectorF)
         Me.Near = Near : Me.Far = Far
      End Sub
      Public Sub New(ByVal x1 As Single, ByVal y1 As Single, ByVal x2 As Single, ByVal y2 As Single)
         MyClass.New(New VectorF(x1, y1), New VectorF(x2, y2))
      End Sub
      Public Shared Function From(ByVal eachMember As Single) As RectF
         Return RectF.From(VectorF.From(eachMember))
      End Function
      Public Shared Function From(ByVal eachMember As VectorF) As RectF
         Return New RectF(eachMember, eachMember)
      End Function
      Public Shared Function From(ByVal rct As RectangleF) As RectF
         Return New RectF(rct.Location, rct.Location + rct.Size)
      End Function
      Public Function Add(ByVal toEach As Single) As RectF
         Return Add(VectorF.From(toEach))
      End Function
      ''' <summary>performs offset </summary>
      Public Function Add(ByVal toEach As VectorF) As RectF
         Return New RectF(Near + toEach, Far + toEach)
      End Function
      ''' <summary>
      ''' performs sophisticated offsets, eg. inflating/deflating (if v1 = -v2)
      ''' </summary>
      Public Function Add(ByVal v1 As VectorF, ByVal v2 As VectorF) As RectF
         Return New RectF(Near + v1, Far + v2)
      End Function
      ''' <summary>
      ''' performs sophisticated offsets, supports compatiblity to RectangleF
      ''' </summary>
      Public Function Add(ByVal r As RectF) As RectF
         Return New RectF(Near + r.Near, Far + r.Far)
      End Function
      Public Function Subtract(ByVal toEach As Single) As RectF
         Return Subtract(VectorF.From(toEach))
      End Function
      ''' <summary>performs offset </summary>
      Public Function Subtract(ByVal toEach As VectorF) As RectF
         Return New RectF(Near - toEach, Far - toEach)
      End Function
      ''' <summary>
      ''' performs offset as well as inflating (use negative Vectors)
      ''' </summary>
      Public Function Subtract(ByVal v1 As VectorF, ByVal v2 As VectorF) As RectF
         Return New RectF(Near - v1, Far - v2)
      End Function
      ''' <summary>
      ''' performs sophisticated offsets, supports compatiblity to RectangleF
      ''' </summary>
      Public Function Subtract(ByVal r As RectF) As RectF
         Return New RectF(Near - r.Near, Far - r.Far)
      End Function
      Public Overrides Function ToString() As String
         Return String.Concat("{", Near, " ", Far, "}")
      End Function
      ''' <remarks>the getter is for compatibility only. better access RectF.Near directly</remarks>
      Public Property Location() As VectorF
         Get
            Return Near
         End Get
         Set(ByVal value As VectorF)
            Far += value - Near : Near = value
         End Set
      End Property
      Public Property Size() As VectorF
         Get
            Return Far - Near
         End Get
         Set(ByVal value As VectorF)
            Far = Near + value
         End Set
      End Property
      Default Public Property Item(ByVal far As Integer) As VectorF
         Get
            If far = 0 Then Return Near
            Return Me.Far
         End Get
         Set(ByVal value As VectorF)
            If far = 0 Then Near = value Else Me.Far = value
         End Set
      End Property
      Default Public Property Item(ByVal far As Integer, ByVal vertical As Integer) As Single
         Get
            If far = 0 Then Return Near(vertical)
            Return Me.Far(vertical)
         End Get
         Set(ByVal value As Single)
            If far = 0 Then Near(vertical) = value Else Me.Far(vertical) = value
         End Set
      End Property
      ''' <summary>
      ''' a none-intersection is indicated by returning a imaginary RectF
      ''' </summary>
      Public Function Intersect(ByVal other As RectF) As RectF
         For i = 0 To 1
            Intersect.Near(i) = Near(i).Max(other.Near(i))
            Intersect.Far(i) = Far(i).Min(other.Far(i))
         Next
      End Function
      ''' <summary>true, if my size has a negative component</summary>
      Public ReadOnly Property IsImaginary() As Boolean
         Get
            If Far.X - Near.X < 0 Then Return True
            Return Far.Y - Near.Y < 0
         End Get
      End Property
      Public Shared Operator -(ByVal r As RectF) As RectF
         Return New RectF(-r.Near, -r.Far)
      End Operator
      Public Shared Operator =(ByVal v1 As RectF, ByVal v2 As RectF) As Boolean
         Return v1.Near = v2.Near AndAlso v1.Far = v2.Far
      End Operator
      Public Shared Operator <>(ByVal v1 As RectF, ByVal v2 As RectF) As Boolean
         Return Not v1 = v2
      End Operator
      Public Overrides Function Equals(ByVal obj As Object) As Boolean
         If TypeOf obj Is RectF OrElse TypeOf obj Is RectangleF Then Return Me = CType(obj, RectF)
      End Function
      Public Overrides Function GetHashCode() As Integer
         Return Near.GetHashCode Xor Far.GetHashCode
      End Function
      Public Shared Widening Operator CType(ByVal other As RectangleF) As RectF
         Return New RectF(other.Location, other.Location + other.Size)
      End Operator
      Public Shared Widening Operator CType(ByVal other As RectF) As RectangleF
         Return New RectangleF(other.Near, other.Far - other.Near)
      End Operator
      Public Overloads Function EachEquals(ByVal amount As Single) As Boolean
         Return Near.EachEquals(amount) AndAlso Far.EachEquals(amount)
      End Function
      ''' <summary>enables Linq-power to Rects elements</summary>
      Public Function GetEnumerator() As IEnumerator(Of VectorF) Implements _
            IEnumerable(Of VectorF).GetEnumerator
         Return DirectCast(New VectorF() {Near, Far}, IEnumerable(Of VectorF)) _
            .GetEnumerator
      End Function

      Private Function GetEnumerator1() As IEnumerator Implements _
            IEnumerable.GetEnumerator
         Return DirectCast(New VectorF() {Near, Far}, IEnumerable(Of VectorF)) _
            .GetEnumerator
      End Function
   End Structure

End Namespace
