﻿Imports System.Runtime.CompilerServices
Imports System.Drawing

Namespace System.Windows.Forms

   Public Module TreeNodeX

      Public Structure ApproachResult
         Public ReadOnly Node As TreeNode
         Public ReadOnly StepsDone As Integer
         Public ReadOnly Segments As IList(Of String)

         Public Sub New( _
               ByVal Node As TreeNode, _
               ByVal Segments As IList(Of String), _
               ByVal StepsDone As Integer)
            Me.Node = Node
            Me.Segments = Segments
            Me.StepsDone = StepsDone
         End Sub

         Public ReadOnly Property IsFullMatch() As Boolean
            Get
               Return Segments.Count = StepsDone
            End Get
         End Property
      End Structure 'ApproachResult

      ''' <summary>
      ''' sucht in Nodes und SubNodes anhand eines Pfades nach dem bezeichneten Node.
      ''' Bei Nichtexistenz des ZielNode den übergeordneten, in den der ZielNode zu erstellen wäre
      ''' </summary>
      ''' <param name="PathSegments">Die Segmente des Pfades</param>
      ''' <returns>ein 
      ''' <see cref="ApproachResult"> ApproachResult, 
      ''' das den gefundenen Node enthält, und die Anzahl abgearbeiteter Segmente
      ''' </see>
      ''' </returns>
      <Extension()> _
      Public Function Approach(ByVal Nodes As TreeNodeCollection, _
            ByVal PathSegments As IList(Of String)) As ApproachResult
         Dim Nd As TreeNode = Nothing, I As Integer
         For I = 0 To PathSegments.Count - 1
            Dim tmp = Nodes.ByText(PathSegments(I))
            If tmp Is Nothing Then Exit For
            Nd = tmp
            Nodes = Nd.Nodes
         Next
         Return New ApproachResult(Nd, PathSegments, I)
      End Function

      <Extension()> _
      Public Function ByText(ByVal Nodes As TreeNodeCollection, ByVal text As String) As TreeNode
         For Each nd As TreeNode In Nodes
            If String.Compare(nd.Text, text, True) = 0 Then Return nd
         Next
         Return Nothing
      End Function

      <Extension()> _
      Public Function Approach(ByVal TV As TreeView, ByVal NodePath As String) As ApproachResult
         Return TV.Nodes.Approach(NodePath.Split(New String() {TV.PathSeparator}, StringSplitOptions.None))
      End Function

      <Extension()> _
      Public Function ParentNodes(ByVal Nd As TreeNode) As TreeNodeCollection
         Dim Parent = Nd.Parent
         Return If(Parent Is Nothing, Nd.TreeView.Nodes, Parent.Nodes)
      End Function

      <Extension()> _
      Public Sub Delete(ByVal Nd As TreeNode, Optional ByVal Recursive As Boolean = False)
         Dim ParentNodes = Nd.ParentNodes
         If Not Recursive Then
            'nur diese Instanz ausschneiden, Children an meinen Parent hängen
            Dim Nodes = Nd.Nodes
            For I = Nodes.Count - 1 To 0 Step -1
               Dim Nd2 = Nodes(I)
               Nodes.RemoveAt(I)
               ParentNodes.Add(Nd2)
            Next
         End If
         Nd.Remove()
      End Sub

      <Extension()> _
      Public Sub EnsureVisibleX( _
               ByVal Nd As TreeNode, _
               ByVal Align As StringAlignment, Optional ByVal Forced As Boolean = False)
         If Nd.IsVisible AndAlso Not Forced Then Return
         'Zeigt den sichtbar zu machenden Node Oben / Unten / Mitten im Treeview an.
         Dim TV = Nd.TreeView
         Select Case Align
            Case StringAlignment.Near
               TV.TopNode = Nd

            Case StringAlignment.Far
               TV.TopNode = TV.Nodes(0)
               Nd.EnsureVisible()

            Case StringAlignment.Center
               Nd.EnsureVisibleX(StringAlignment.Far)
               With Nd.Bounds
                  Dim Nd2 As TreeNode = TV.GetNodeAt(1, .Bottom + .Height - TV.Height \ 2)
                  If Nd2 IsNot Nothing Then TV.TopNode = Nd2
               End With

         End Select
      End Sub

   End Module

End Namespace