﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Diagnostics
Imports System.Collections

Namespace System
   <Serializable()> _
   <DebuggerDisplay("{me.Start} {End}")> _
   Public Structure IntRange
      Implements IEnumerable(Of Integer)

      Public Property Start As Integer
      Public [End] As Integer

      'comparer support Sorting and binary Search
      Public Shared ReadOnly StartComparer As IComparer(Of IntRange) = (New Comparison(Of IntRange)(Function(r1, r2) r1.Start.CompareTo(r2.Start))).ToComparer()
      Public Shared ReadOnly EndComparer As IComparer(Of IntRange) = (New Comparison(Of IntRange)(Function(r1, r2) r1.End.CompareTo(r2.End))).ToComparer()
      Public Shared Empty As IntRange = IntRange.From(0)

      Public Sub New(ByVal Start As Integer, ByVal [end] As Integer)
         Me.Start = Start
         Me.End = [end]
      End Sub
      Public Sub New(ByVal [each] As Integer)
         Me.Start = [each]
         Me.End = [each]
      End Sub
      Public Shared Function From(ByVal start As Integer) As IntRange
         Return New IntRange(start, start)
      End Function
      Public Shared Function From(ByVal start As Integer, ByVal [end] As Integer) As IntRange
         Return New IntRange(start, [end])
      End Function
      Public Shared Function FromLength(ByVal start As Integer, ByVal length As Integer) As IntRange
         Return New IntRange(start, start + length)
      End Function
      Public Function Add(ByVal [each] As Integer) As IntRange
         Return New IntRange(Me.Start + [each], Me.end + [each])
      End Function
      Public Function Add(ByVal Start As Integer, ByVal [end] As Integer) As IntRange
         Return New IntRange(Me.Start + Start, Me.End + [end])
      End Function
      Public Function Subtract(ByVal [each] As Integer) As IntRange
         Return New IntRange(Me.Start - [each], Me.end - [each])
      End Function
      Public Function Subtract(ByVal Start As Integer, ByVal [end] As Integer) As IntRange
         Return New IntRange(Me.Start - Start, Me.End - [end])
      End Function
      Default Public Property Item(ByVal end__1 As Integer) As Integer
         Get
            Return If(end__1 = 0, Me.Start, Me.end)
         End Get
         Set(ByVal value As Integer)
            If end__1 = 0 Then
               Me.Start = value
            Else
               Me.end = value
            End If
         End Set
      End Property
      Public Overrides Function ToString() As String
         Return String.Concat("{", Me.Start, " ", Me.end, "}")
      End Function
      Public Property Length() As Integer
         Get
            Return Me.end - Me.Start
         End Get
         Set(ByVal value As Integer)
            Me.end = Me.Start + value
         End Set
      End Property

      Public Function Union(ByVal Start As Integer, ByVal [end] As Integer) As IntRange
         Return New IntRange(Math.Min(Me.Start, Math.Min(Me.End, Math.Min(Start, [end]))), Math.Max(Me.Start, Math.Max(Me.End, Math.Max(Start, [end]))))
      End Function
      Public Function Union() As IntRange
         Return New IntRange(Math.Min(Me.Start, Me.end), Math.Max(Me.Start, Me.end))
      End Function
      Public Function Union(ByVal other As IntRange) As IntRange
         Return Union(other.Start, other.End)
      End Function
      ''' <summary>meaningful suitable ome.end ordered Lists</summary>
      Public Function UnionFirstLast(ByVal list As IList(Of IntRange)) As IntRange
         If list.Count = 0 Then
            Return Union()
         End If
         Return Union(list(0).Start, list(list.Count - 1).End)
      End Function
      Public Function Intersect(ByVal other As IntRange) As IntRange
         Return New IntRange(Math.Max(Me.Start, other.Start), Math.Min(Me.end, other.End))
      End Function
      <Obsolete("whats the use?")> _
      Public Function Shift(ByVal other As IntRange) As IntRange
         Return Shift(other.Start, other.Length)
      End Function
      <Obsolete("whats the use?")> _
      Public Function Shift(ByVal Start As Integer, ByVal length As Integer) As IntRange
         If Me.end < Start Then
            Return Me
         End If
         If Me.Start > Start Then
            Return New IntRange(Me.Start + length, Me.end + length)
         End If
         Return New IntRange(Me.Start, Me.end + length)
      End Function
      Public Shared Operator =(ByVal v1 As IntRange, ByVal v2 As IntRange) As Boolean
         Return v1.Start = v2.Start AndAlso v1.End = v2.End
      End Operator
      Public Shared Operator <>(ByVal v1 As IntRange, ByVal v2 As IntRange) As Boolean
         Return Not (v1 = v2)
      End Operator
      Public Overrides Function Equals(ByVal obj As Object) As Boolean
         If Not (TypeOf obj Is IntRange) Then
            Return False
         End If
         Return Me = CType(obj, IntRange)
      End Function
      Public Overrides Function GetHashCode() As Integer
         Return Me.Start Xor Me.end
      End Function
      Public Function EachEquals(ByVal amount As Integer) As Boolean
         Return Me.Start = amount AndAlso Me.end = amount
      End Function
      Public Sub SetEach(ByVal amount As Integer)
         Me.Start = amount
         Me.End = amount
      End Sub
      ''' <summary>enables Linq-power to IntRange Elements</summary>
      Public Function GetEnumerator() As IEnumerator(Of Integer) Implements IEnumerable(Of Integer).GetEnumerator
         Return DirectCast(New Integer() {Me.Start, Me.end}, IEnumerable(Of Integer)).GetEnumerator()
      End Function
      Private Function IEnumerable_GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
         Return DirectCast(New Integer() {Me.Start, Me.end}, IEnumerable(Of Integer)).GetEnumerator()
      End Function
   End Structure
End Namespace
