Imports System.Windows.Forms
Imports System.ComponentModel
Imports System.Data.Common
Imports System.Text
Imports System.Text.RegularExpressions

Namespace System.Data

   Public Class DatasetAdapter : Implements IInitFromTableAdapters, IInitFromTables

      Private _FillAdapters As IList(Of TableAdapter) 'werden durch Load() befllt
      Private _IncrementAdapters As IList(Of TableAdapter) 'werden durch explizite Aufrufe befllt
      Private _UpdateAdapters As IList(Of TableAdapter) 'Summe der vorigen - Save() updated alle
      Private _Dataset As DataSet
      Private WithEvents _Connection As DbConnection

      Public Property Connection() As DbConnection
         Get
            If _Connection Is Nothing Then
               _Connection = _UpdateAdapters(0).AllCommands.First().Connection
               If _UpdateAdapters.Count > 1 Then DeployConnection(_Connection, _UpdateAdapters.Skip(1))
            End If
            Return _Connection
         End Get
         Set(ByVal value As DbConnection)
            If _Connection Is value Then Return
            _Connection = value
            DeployConnection(value, _UpdateAdapters)
         End Set
      End Property

      Private Sub DeployConnection( _
            ByVal newCon As DbConnection, ByVal adapters As IEnumerable(Of TableAdapter))
         'jeder generierte TableAdapter hat eine eigene Connection, und all seine Commands verweisen darauf. Hier werden alle Verweise disposed (also teilw. redundant disposed), und durch newCon ersetzt
         For Each cmd In adapters.SelectMany(Function(ta) ta.AllCommands)
            cmd.Connection.Dispose()
            cmd.Connection = newCon
         Next
      End Sub

      Public Sub Save()
         'Lschweitergabe zur Erhaltung der Datenkonsistenz: Lschen einer bergeordneten
         ' Datarow lscht alle ChildRows mit
         'Problem beim Update: Update ich den DELETE einer bergeordneten Datarow, so wendet
         ' auch die Datenbank Lschweitergabe auf ihre Daten an. Update ich danach die DELETEs
         ' der ChildRows, so wirft die DB eine Exception, denn bei ihr sind diese Daten schon weg.
         '(Hier vorgetragene) Lsung: Zunchst nur die DELETEs aller bergeordneter Rows des
         ' ganzen Datasets updaten (Top -> Down). Dabei temporr fr alle Datarelations des
         ' Datasets AcceptRejectRule.Cascade einstellen. Wenn nun die bergeordneten geupdatet
         ' sind, und (intern) ihr AcceptChanges() aufrufen, akzeptieren die ChildRows auch.
         'Eine akzeptierte (Child-)Datarow gibt mit DataRowState.Unchanged an, da sie in 
         'bereinstimmung mit der Datenbank ist, und vom Update auszunehmen.
         Using Connection.EnterOpen
            SetAcceptReject(AcceptRejectRule.Cascade)
            For Each adp In _UpdateAdapters
               If adp.Table.ChildRelations.Count > 0 Then
                  Dim deletes = adp.Table.Select("", "", DataViewRowState.Deleted)
                  If deletes.Count > 0 Then adp.Adapter.Update(deletes)
               End If
            Next
            SetAcceptReject(AcceptRejectRule.None)
            'INSERTs und UPDATEs normal updaten
            For Each adp In _UpdateAdapters
               adp.Adapter.Update(_Dataset)
            Next
         End Using
      End Sub

      Private Sub SetAcceptReject(ByVal Rule As AcceptRejectRule)
         For Each adp In _UpdateAdapters
            For Each Rel As DataRelation In adp.Table.ChildRelations
               Rel.ChildKeyConstraint.AcceptRejectRule = Rule
            Next
         Next
      End Sub

		Public Sub Load()
         Using Connection.EnterOpen
            For Each adp In _FillAdapters : adp.Enable = False : Next
            For Each adp In _FillAdapters : adp.Fill() : Next
            For Each adp In _FillAdapters : adp.Enable = True : Next
         End Using
      End Sub

      Public Function FillByParentRow(ByVal parentRow As DataRow, ByVal table As DataTable) As Integer
         Return FillByParentRow(parentRow, parentRow.Table.GetPathTo(table).ToArray)
      End Function

      Public Function FillByParentRowByTableChain(ByVal parentRow As DataRow, ByVal ParamArray tableChain() As DataTable) As Integer
         tableChain = NewArray(parentRow.Table).Concat(tableChain).ToArray
         Dim RelationPath = New List(Of DataRelation)
         For i = 0 To tableChain.Length - 2
            Dim child = tableChain(i + 1)
            RelationPath.Add(tableChain(i).ChildRelations.Cast(Of DataRelation).First(Function(rl) rl.ChildTable Is child))
         Next
         Return FillByParentRow(parentRow, RelationPath)
      End Function

      Private Function FillByParentRow(ByVal parentRow As DataRow, ByVal RelationPath As IList(Of DataRelation)) As Integer
         Dim table = RelationPath.Last.ChildTable
         Dim primCol = RelationPath(0).ParentColumns(0)
         Dim joiner = primCol.JoinWith(primCol.ColumnName & "=?", RelationPath(0).ChildColumns(0))
         For Each rl In RelationPath.Skip(1)
            joiner.AddJoin(rl.ParentColumns(0), rl.ChildColumns(0))
         Next
         Using Connection.EnterOpen
            If _Dataset.HasChanges Then Save()
            Return FillFromWhere(table, joiner.Sql, parentRow(primCol))
         End Using
      End Function

      Private Shared _rgxSelector As New Regex("SELECT(\s+[\s\S]*?)\s+FROM\s+", RegexOptions.IgnoreCase)
      Private Shared _rgxWordStart As New Regex("(?<=\s)(?=\S)")

      'Regex-Funktionen
      ' (\s+As\s+): SQL-Umbenennungs-Indikator, [\w\.]+: ggfs. vollqualifizierter Bezeichner,
      ' ([<=>]): Comparator, \?: Param-Placeholder
      Private Shared _rgxGetSqlWords As New Regex("(?<Rename>\s+As\s+)|[\w\.]+|(?<Compare>[<=>])|\?", RegexOptions.IgnoreCase)
      Public Function FillFromWhere(ByVal table As DataTable, ByVal sqlFromWhere As String, ByVal paramValues As IList(Of Object), Optional ByVal clearBeforeFill As Boolean = True) As Integer
         Dim adp = GetAdapter(table.TableName)
         'vollqualifizierte Namen im Select-Abschnitt gewhrleisten
         Dim selector = _rgxSelector.Matches(adp.AllCommands(0).CommandText)(0).Groups(1).Value
         If Not selector.Contains("."c) Then selector = _rgxWordStart.Replace(selector, table.TableName & ".")
         Using cmd = Connection.CreateCommand
            cmd.CommandText.Set("SELECT ", selector, " FROM ", sqlFromWhere)
            Dim mts = _rgxGetSqlWords.Matches(sqlFromWhere)
            Dim iParam As Integer = 0
            Dim renames As New Dictionary(Of String, String)
            For i = 0 To mts.Count - 1
               'Schleife mit 2 Analysen: a) Umbenennungen merken, b) DBParameter erstellen aus Parameter-Platzhalter und paramValues
               If mts(i).Groups("Rename").Success Then
                  renames.Add(mts(i + 1).Value, mts(i - 1).Value)
               ElseIf mts(i).Value = "?" Then
                  'Der zu comparende Parameter ist entweder 2 Matches vor oder hinter dem Parameter-Platzhalter
                  Dim iParamMatch = If(mts(i - 1).Groups("Compare").Success, i - 2, i + 2)
                  Dim paramParts = mts(iParamMatch).Value.Split("."c)
                  If paramParts.Length < 2 Then paramParts = New String() {table.TableName, paramParts(0)}
                  cmd.Parameters.Add(CreateParam(paramParts, paramValues(iParam), renames))
                  iParam += 1
               End If
            Next
            If iParam <> paramValues.Count Then Throw Me.Exception( _
               "Es wurde eine andere Anzahl Werte bergeben, als Platzhalter im Sql-String angegeben sind")
            Return adp.FillTable(cmd, clearBeforeFill)
         End Using
      End Function

      Public Function FillFromWhere(ByVal table As DataTable, ByVal sqlFromWhere As String, ByVal ParamArray values As Object()) As Integer
         Return FillFromWhere(table, sqlFromWhere, DirectCast(values, IList(Of Object)))
      End Function

      Public Function GetAdapter(ByVal table As DataTable) As TableAdapter
         Return _UpdateAdapters.First(Function(a) a.Table Is table)
      End Function

      Public ReadOnly Property SelectCmd(ByVal table As DataTable) As DbCommand
         Get
            Return _UpdateAdapters.First(Function(a) a.Table Is table).AllCommands(0)
         End Get
      End Property

      Public ReadOnly Property InsertCmd(ByVal table As DataTable) As DbCommand
         Get
            Return _UpdateAdapters.First(Function(a) a.Table Is table).AllCommands(2)
         End Get
      End Property

      Private Function GetAdapter(ByVal tableName As String) As TableAdapter
         Return _UpdateAdapters.First(Function(a) a.Table.TableName = tableName)
      End Function

      Private Function CreateParam(ByVal fullname As String(), ByVal value As Object, ByVal renames As Dictionary(Of String, String)) As DbParameter
         If renames.ContainsKey(fullname(0)) Then fullname(0) = renames(fullname(0))
         Dim adp = GetAdapter(fullname(0))
         CreateParam = adp.Adapter.InsertCommand.Parameters(fullname(1)).Clone
         CreateParam.Value = value
      End Function

      Public Enum BsPlug
         [On]
         Off
         ExclusiveOn
      End Enum


      Private WithEvents _SourceInfos As New BindingSourceInfos

      Private Sub _SourceInfos_TablesNeedsFill(ByVal sender As Object, ByVal e As BindingSourceInfos.EventArg) Handles _SourceInfos.TablesNeedsFill
         For Each tb In e.ChildTables
            FillByParentRowByTableChain(e.ParentRow, tb)
         Next
      End Sub

      Public Sub Plug(ByVal bs As BindingSource, ByVal plug As BsPlug)
         _SourceInfos(bs).Plug = plug <> BsPlug.Off
      End Sub

   End Class

End Namespace